import { routerRedux } from 'dva/router';
import { logout, fetchModuleInfos } from '../../services/main';
import { fetchDomains, switchDomain, currentDomain } from '../../services/domain';
import { cookie, storage } from '../../utils/config';
import { setModule, getModule, getPath, foreachModule } from '../../data/modules';
import { setCookie, delCookie, getLocalStorge, setLocalStorge, delLocalStorge, fullPath } from '../../utils/helper';

const createMenus = () => {
  const menus = [];
  const makePath = (_menus, path) => {
    if (path.length === 0) {
      return null;
    }
    const [cur, ...rest] = path;
    const find = _menus.filter(menu => menu.name === `${cur}`).pop();
    if (find) {
      if (rest.length === 0) {
        return find;
      } else {
        return makePath(find.children, rest);
      }
    } else {
      let container = _menus;
      for (const id of path) {
        const m = getModule(id);
        const menu = {
          name: `${m.id}`,
          text: m.name,
          children: [],
        };
        container.push(menu);
        container = menu.children;
      }
    }
  };
  const makeIcon = (menu) => {
    const children = menu.children;
    if (children.length === 0) {
      menu.icon = 'file'; // eslint-disable-line no-param-reassign
    } else {
      menu.icon = 'folder'; // eslint-disable-line no-param-reassign
      for (const child of children) {
        makeIcon(child);
      }
    }
  };
  foreachModule((module) => {
    const path = getPath(module.id);
    makePath(menus, path);
  });
  for (const menu of menus) {
    makeIcon(menu);
  }
  return menus;
};

export default {

  namespace: 'main',

  state: {
    user: '',
    domain: '',
    domainList: [],
    menus: [],
  },

  subscriptions: {
    setup({ dispatch, history }) {  // eslint-disable-line
    },
  },

  effects: {
    *fetchDomain(action, { put, call }) {
      let domain = getLocalStorge(storage.domainName);
      if (!domain) {
        const { id, name } = yield call(currentDomain);
        setCookie(cookie.domain, id);
        setLocalStorge(storage.domainName, name);
        domain = name;
      }
      yield put({ type: 'switchDomainSuccess', payload: domain });
    },
    *fetchDomains(action, { put, call }) {
      const list = yield call(fetchDomains);
      yield put({ type: 'queryDomainsSuccess', payload: list });
    },
    *switchDomain({ payload: domainId }, { put, call }) {
      yield call(switchDomain, domainId);
      const { id, name } = yield call(currentDomain);
      setCookie(cookie.domain, id);
      setLocalStorge(storage.domainName, name);
      yield put({ type: 'switchDomainSuccess', payload: name });
      yield put(routerRedux.push(fullPath('/main')));
    },
    *fetchUser(action, { put }) {
      const user = getLocalStorge(storage.userName);
      if (!user) {
        yield put(routerRedux.push(fullPath('/login')));
      }
      yield put({ type: 'queryUserSuccess', payload: user });
    },
    *fetchModules(action, { put, call }) {
      const modules = yield call(fetchModuleInfos);
      setModule(modules);
      yield put({ type: 'queryMenusSuccess', payload: createMenus(modules) });
    },
    *logout(action, { put, call }) {
      yield call(logout);
      delLocalStorge(storage.domainName);
      delLocalStorge(storage.userName);
      delCookie(cookie.domain);
      delCookie(cookie.user);
      delCookie(cookie.token);
      yield put(routerRedux.push(fullPath('/login')));
    },
  },

  reducers: {
    queryUserSuccess(state, { payload: user }) {
      return {
        ...state,
        user,
      };
    },
    queryDomainsSuccess(state, { payload: list }) {
      return {
        ...state,
        domainList: list,
      };
    },
    switchDomainSuccess(state, { payload: domain }) {
      return {
        ...state,
        domain,
      };
    },
    queryMenusSuccess(state, { payload: menus }) {
      return {
        ...state,
        menus,
      };
    },
  },

};
